﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using EPiServer.Web.PropertyControls;
using System.ComponentModel;
using System.Reflection;
using System.Web.UI.WebControls;
using EPiServer.Core;
using EPiServer.PlugIn;
using EPiServer.Core.PropertySettings;

namespace EPiServer.Composer.Functions
{
    public enum SortOrder
    {
        [Description("Create Date (Newest first)")]
        CreateDateNew,
        [Description("Create Date (Oldest first)")]
        CreateDateOld,
        [Description("Publish Date (Newest first)")]
        PublishDateNew,
         [Description("Publish Date (Oldest first)")]
        PublishDateOld,
        Alphabetical,
         [Description("Change Date (Newest first)")]
        ChangeDateNew
    }
    public static class Enumeration
    {
        public static IEnumerable<T> EnumToList<T>()
        {
            Type enumType = typeof(T);

            // Can't use generic type constraints on value types,
            // so have to do check like this
            if (enumType.BaseType != typeof(Enum))
                throw new ArgumentException("T must be of type System.Enum");

            Array enumValArray = Enum.GetValues(enumType);
            List<T> enumValList = new List<T>(enumValArray.Length);

            foreach (int val in enumValArray)
            {
                enumValList.Add((T)Enum.Parse(enumType, val.ToString()));
            }

            return enumValList;
        }

        public static string GetEnumDescription(Enum value)
        {
            FieldInfo fi = value.GetType().GetField(value.ToString());

            DescriptionAttribute[] attributes =
                (DescriptionAttribute[])fi.GetCustomAttributes(
                typeof(DescriptionAttribute),
                false);

            if (attributes != null &&
                attributes.Length > 0)
                return attributes[0].Description;
            else
                return value.ToString();
        }
    }

    [Serializable]
    [PageDefinitionTypePlugIn(DisplayName = "SortingProperty",
        Description = "Pick a sorting order for category listing",
        LanguagePath = "")]
    public class SortingProperty : PropertyString

    {
        public override IPropertyControl CreatePropertyControl()
        {
            return new SortingPropertyControl();
        }

    }

    
    public class SortingPropertyControl : PropertySelectControlBase
    {

        protected override void SetupEditControls()
        {
            foreach (var item in Enumeration.EnumToList <SortOrder>())
            {

                var listItem = new ListItem(Enumeration.GetEnumDescription(item), item.ToString());
                if(this.ToString() == item.ToString())
                {
                    listItem.Selected = true;
                }

                EditControl.Items.Add(listItem);
            }   
        }


        public override void ApplyEditChanges()
        {

            if (EditControl.SelectedValue != null) 
            {
                SetValue(EditControl.SelectedValue);
            }
            else SetValue(null);
        }


     
    }


}